@@ -1,12 +1,11 @@
-#!/usr/bin/python
+#!/usr/bin/env python3
 
 
 '''
-Main import file used when 'import meraki-cli'
+Main import file used when 'import meraki_cli'
 '''
 
 
-from . import __main__
 from . import __version__
 
 
