@@ -24,8 +24,15 @@ OUTPUT = [
     },
 ]
 
+BAD_TRANSLATION = 'changed-changeme'  # Uses a minus 'accidently'
+
 
 class TestTranslations(unittest.TestCase):
 
     def testTranslation(self):
         assert _translate_input(INPUT, ['changed=changeme']) == OUTPUT
+
+    def testTranslationBadFormatLogThenExit(self):
+        with self.assertLogs(level='CRITICAL'):
+            with self.assertRaises(SystemExit):
+                _translate_input(INPUT, BAD_TRANSLATION) == OUTPUT
